@@ -1,6 +1,6 @@
 import urllib.parse
 
-from flask import request, jsonify, current_app
+from flask import request, jsonify, current_app, redirect, make_response
 from flask_restful import Resource
 from flask_principal import (
     Identity,
@@ -14,11 +14,13 @@ from flask_jwt_extended import (
     set_access_cookies,
     set_refresh_cookies,
     unset_jwt_cookies,
+    unset_refresh_cookies,
     get_jwt,
 )
 
 from sqlalchemy.exc import OperationalError, TimeoutError
 from babel.dates import format_datetime
+from saml2 import entity
 
 from zou.app import app, config
 from zou.app.mixin import ArgsMixin
@@ -77,7 +79,9 @@ class AuthenticatedResource(Resource):
             description: Person not found
         """
         person = persons_service.get_current_user(relations=True)
-        organisation = persons_service.get_organisation()
+        organisation = persons_service.get_organisation(
+            sensitive=permissions.has_admin_permissions()
+        )
         return {
             "authenticated": True,
             "user": person,
@@ -217,7 +221,6 @@ class LoginResource(Resource, ArgsMixin):
                     "identity_type": "person",
                 },
             )
-            auth_service.register_tokens(app, access_token, refresh_token)
             identity_changed.send(
                 current_app._get_current_object(),
                 identity=Identity(user["id"], "person"),
@@ -227,15 +230,21 @@ class LoginResource(Resource, ArgsMixin):
                 "HTTP_X_REAL_IP", request.remote_addr
             )
 
+            organisation = persons_service.get_organisation(
+                sensitive=user["role"] != "admin"
+            )
+
+            response = jsonify(
+                {
+                    "user": user,
+                    "organisation": organisation,
+                    "login": True,
+                    "access_token": access_token,
+                    "refresh_token": refresh_token,
+                }
+            )
+
             if is_from_browser(request.user_agent):
-                organisation = persons_service.get_organisation()
-                response = jsonify(
-                    {
-                        "user": user,
-                        "organisation": organisation,
-                        "login": True,
-                    }
-                )
                 set_access_cookies(response, access_token)
                 set_refresh_cookies(response, refresh_token)
                 events_service.create_login_log(user["id"], ip_address, "web")
@@ -243,12 +252,6 @@ class LoginResource(Resource, ArgsMixin):
                 events_service.create_login_log(
                     user["id"], ip_address, "script"
                 )
-                response = {
-                    "login": True,
-                    "user": user,
-                    "access_token": access_token,
-                    "refresh_token": refresh_token,
-                }
             current_app.logger.info(f"User {email} is logged in.")
             return response
         except WrongUserException:
@@ -383,10 +386,10 @@ class RefreshTokenResource(Resource):
                 "identity_type": "person",
             },
         )
-        auth_service.register_tokens(app, access_token)
         if is_from_browser(request.user_agent):
             response = jsonify({"refresh": True})
             set_access_cookies(response, access_token)
+            unset_refresh_cookies(response)
         else:
             return {"access_token": access_token}
 
@@ -733,6 +736,11 @@ class ResetPasswordResource(Resource, ArgsMixin):
 
         try:
             user = persons_service.get_person_by_email(args["email"])
+            if not user["active"]:
+                return (
+                    {"error": True, "message": "This user is inactive."},
+                    400,
+                )
         except PersonNotFoundException:
             return (
                 {"error": True, "message": "Email not listed in database."},
@@ -1328,3 +1336,128 @@ class RecoveryCodesResource(Resource, ArgsMixin):
                 },
                 400,
             )
+
+
+class SAMLSSOResource(Resource, ArgsMixin):
+    """
+    Resource to allow a user to login with SAML SSO.
+    """
+
+    def post(self):
+        """
+        Resource to allow a user to login with SAML SSO.
+        ---
+        description: ""
+        tags:
+            - Authentication
+        responses:
+          302:
+            description: Login successful, redirect to the home page.
+          400:
+            description: Wrong parameter
+        """
+        if not config.SAML_ENABLED:
+            return {"error": "SAML is not enabled."}, 400
+        authn_response = current_app.extensions[
+            "saml_client"
+        ].parse_authn_request_response(
+            request.form["SAMLResponse"], entity.BINDING_HTTP_POST
+        )
+        authn_response.get_identity()
+        email = authn_response.get_subject().text
+        person_info = {
+            k: (
+                " ".join(v)
+                if isinstance(v, list) and k in ["first_name", "last_name"]
+                else v
+            )
+            for k, v in authn_response.ava.items()
+            if k
+            in [
+                "first_name",
+                "last_name",
+                "phone",
+                "role",
+                "departments",
+                "studio_id",
+                "active",
+            ]
+        }
+        try:
+            user = persons_service.get_person_by_email(email)
+            for k, v in person_info.items():
+                if user.get(k) != v:
+                    persons_service.update_person(
+                        user["id"], person_info, bypass_protected_accounts=True
+                    )
+                    break
+        except PersonNotFoundException:
+            user = persons_service.create_person(
+                email, "default".encode("utf-8"), **person_info
+            )
+
+        response = make_response(
+            redirect(f"{config.DOMAIN_PROTOCOL}://{config.DOMAIN_NAME}")
+        )
+
+        if user["active"]:
+            access_token = create_access_token(
+                identity=user["id"],
+                additional_claims={
+                    "identity_type": "person",
+                },
+            )
+            refresh_token = create_refresh_token(
+                identity=user["id"],
+                additional_claims={
+                    "identity_type": "person",
+                },
+            )
+            identity_changed.send(
+                current_app._get_current_object(),
+                identity=Identity(user["id"], "person"),
+            )
+
+            ip_address = request.environ.get(
+                "HTTP_X_REAL_IP", request.remote_addr
+            )
+
+            set_access_cookies(response, access_token)
+            set_refresh_cookies(response, refresh_token)
+            events_service.create_login_log(user["id"], ip_address, "web")
+
+        return response
+
+
+class SAMLLoginResource(Resource, ArgsMixin):
+    """
+    Resource to allow a user to login with SAML SSO.
+    """
+
+    def get(self):
+        """
+        Resource to allow a user to login with SAML SSO.
+        ---
+        description: ""
+        tags:
+            - Authentication
+        responses:
+          302:
+            description: Redirect to the SAML IDP.
+          400:
+            description: Wrong parameter.
+        """
+        if not config.SAML_ENABLED:
+            return {"error": "SAML is not enabled."}, 400
+        _, info = current_app.extensions[
+            "saml_client"
+        ].prepare_for_authenticate()
+
+        redirect_url = None
+
+        # Select the IdP URL to send the AuthN request to
+        for key, value in info["headers"]:
+            if key == "Location":
+                redirect_url = value
+
+        return redirect(redirect_url, code=302)
