@@ -11,7 +11,7 @@ class EventsResource(BaseModelsResource):
         if query is None:
             query = self.model.query
 
-        return self.model.serialize_list(
+        return self.serialize_list(
             query.limit(1000).all(), relations=relations
         )
 
