@@ -29,113 +29,13 @@ from zou.app.services.exception import (
     WrongParameterException,
     UnactiveUserException,
     TwoFactorAuthenticationNotEnabledException,
+    PersonInProtectedAccounts,
 )
 from zou.app.services.auth_service import (
     disable_two_factor_authentication_for_person,
 )
 
 
-class NewPersonResource(Resource, ArgsMixin):
-    """
-    Create a new user in the database.
-    """
-
-    # TODO: Remove this route in the future.
-    @jwt_required()
-    @permissions.require_admin
-    def post(self):
-        """
-        Create a new user in the database.
-        This route will be removed in the future.
-        Please use POST /data/persons instead.
-        ---
-        tags:
-        - Persons
-        description: Set password to None if not provided.
-                     User role can be set but only admins can create admin users.
-        parameters:
-          - in: formData
-            name: email
-            required: True
-            type: string
-            format: email
-            x-example: admin@example.com
-          - in: formData
-            name: phone
-            required: False
-            type: integer
-            x-example: 06 12 34 56 78
-          - in: formData
-            name: role
-            required: False
-            type: string
-            x-example: user
-          - in: formData
-            name: first_name
-            required: True
-            type: string
-          - in: formData
-            name: last_name
-            required: False
-            type: string
-        responses:
-            201:
-                description: User created
-        """
-        data = self.get_arguments()
-
-        if not data["is_bot"] and persons_service.is_user_limit_reached():
-            return {
-                "error": True,
-                "message": "User limit reached.",
-                "limit": config.USER_LIMIT,
-            }, 400
-        else:
-            if data["password"] is not None:
-                data["password"] = auth.encrypt_password(data["password"])
-            person = persons_service.create_person(
-                data["email"],
-                data["password"],
-                data["first_name"],
-                data["last_name"],
-                data["phone"],
-                role=data["role"],
-                desktop_login=data["desktop_login"],
-                departments=data["departments"],
-                is_bot=data["is_bot"],
-                expiration_date=data["expiration_date"],
-                active=data["active"],
-            )
-        return person, 201
-
-    def get_arguments(self):
-        args = self.get_args(
-            [
-                {
-                    "name": "email",
-                    "help": "The email is required.",
-                    "required": True,
-                },
-                {
-                    "name": "first_name",
-                    "help": "The first name is required.",
-                    "required": True,
-                },
-                ("last_name", ""),
-                ("phone", ""),
-                ("role", "user"),
-                ("desktop_login", ""),
-                {"name": "departments", "action": "append"},
-                "password",
-                {"name": "is_bot", "default": False, "type": bool},
-                {"name": "expiration_date", "default": None, "type": str},
-                {"name": "active", "default": True, "type": bool},
-            ]
-        )
-
-        return args
-
-
 class DesktopLoginsResource(Resource, ArgsMixin):
     """
     Allow to create and retrieve desktop login logs. Desktop login logs can only
@@ -407,7 +307,7 @@ class PersonDurationTimeSpentsResource(Resource, ArgsMixin):
                         raise permissions.PermissionDenied
                     if permissions.has_supervisor_permissions():
                         department_ids = persons_service.get_current_user(
-                            True
+                            relations=True
                         )["departments"]
                 else:
                     raise permissions.PermissionDenied
@@ -632,7 +532,24 @@ class PersonDayTimeSpentsResource(PersonDurationTimeSpentsResource):
             abort(404)
 
 
-class PersonMonthQuotaShotsResource(Resource, ArgsMixin):
+class PersonQuotaMixin:
+
+    def get_quota_arguments(self):
+        project_id = self.get_project_id()
+        task_type_id = self.get_task_type_id()
+        count_mode = self.get_text_parameter("count_mode", default="weigthed")
+        if count_mode not in ["weighted", "weighteddone", "feedback", "done"]:
+            raise WrongParameterException(
+                "count_mode must be equal to weighted, weigtheddone, feedback"
+                ", or done"
+            )
+        feedback = "done" not in count_mode
+        weighted = "weighted" in count_mode
+
+        return (project_id, task_type_id, count_mode, feedback, weighted)
+
+
+class PersonMonthQuotaShotsResource(Resource, ArgsMixin, PersonQuotaMixin):
     """
     Get ended shots used for quota calculation of this month.
     """
@@ -663,6 +580,12 @@ class PersonMonthQuotaShotsResource(Resource, ArgsMixin):
             x-example: 07
             minimum: 1
             maximum: 12
+          - in: query
+            name: count_mode
+            required: True
+            type: string
+            enum: [weighted, weigtheddone, feedback, done]
+            x-example: weighted
         responses:
             200:
                 description: Ended shots used for quota calculation of this month
@@ -670,10 +593,11 @@ class PersonMonthQuotaShotsResource(Resource, ArgsMixin):
                 description: Wrong date format
         """
         user_service.check_person_is_not_bot(person_id)
-        project_id = self.get_project_id()
-        task_type_id = self.get_task_type_id()
         user_service.check_person_access(person_id)
-        weighted = self.get_bool_parameter("weighted", default="true")
+        (project_id, task_type_id, count_mode, feedback, weighted) = (
+            self.get_quota_arguments()
+        )
+
         try:
             return shots_service.get_month_quota_shots(
                 person_id,
@@ -682,12 +606,13 @@ class PersonMonthQuotaShotsResource(Resource, ArgsMixin):
                 project_id=project_id,
                 task_type_id=task_type_id,
                 weighted=weighted,
+                feedback=feedback,
             )
         except WrongDateFormatException:
             abort(404)
 
 
-class PersonWeekQuotaShotsResource(Resource, ArgsMixin):
+class PersonWeekQuotaShotsResource(Resource, ArgsMixin, PersonQuotaMixin):
     """
     Get ended shots used for quota calculation of this week.
     """
@@ -718,6 +643,12 @@ class PersonWeekQuotaShotsResource(Resource, ArgsMixin):
             x-example: 35
             minimum: 1
             maximum: 52
+          - in: query
+            name: count_mode
+            required: True
+            type: string
+            enum: [weighted, weigtheddone, feedback, done]
+            x-example: weighted
         responses:
             200:
                 description: Ended shots used for quota calculation of this week
@@ -725,10 +656,11 @@ class PersonWeekQuotaShotsResource(Resource, ArgsMixin):
                 description: Wrong date format
         """
         user_service.check_person_is_not_bot(person_id)
-        project_id = self.get_project_id()
-        task_type_id = self.get_task_type_id()
         user_service.check_person_access(person_id)
-        weighted = self.get_bool_parameter("weighted", default="true")
+        (project_id, task_type_id, count_mode, feedback, weighted) = (
+            self.get_quota_arguments()
+        )
+
         try:
             return shots_service.get_week_quota_shots(
                 person_id,
@@ -737,12 +669,13 @@ class PersonWeekQuotaShotsResource(Resource, ArgsMixin):
                 project_id=project_id,
                 task_type_id=task_type_id,
                 weighted=weighted,
+                feedback=feedback,
             )
         except WrongDateFormatException:
             abort(404)
 
 
-class PersonDayQuotaShotsResource(Resource, ArgsMixin):
+class PersonDayQuotaShotsResource(Resource, ArgsMixin, PersonQuotaMixin):
     """
     Get ended shots used for quota calculation of this day.
     """
@@ -780,6 +713,12 @@ class PersonDayQuotaShotsResource(Resource, ArgsMixin):
             x-example: 12
             minimum: 1
             maximum: 31
+          - in: query
+            name: count_mode
+            required: True
+            type: string
+            enum: [weighted, weigtheddone, feedback, done]
+            x-example: weighted
         responses:
             200:
                 description: Ended shots used for quota calculation of this day
@@ -787,10 +726,11 @@ class PersonDayQuotaShotsResource(Resource, ArgsMixin):
                 description: Wrong date format
         """
         user_service.check_person_is_not_bot(person_id)
-        project_id = self.get_project_id()
-        task_type_id = self.get_task_type_id()
         user_service.check_person_access(person_id)
-        weighted = self.get_bool_parameter("weighted", default="true")
+        (project_id, task_type_id, count_mode, feedback, weighted) = (
+            self.get_quota_arguments()
+        )
+
         try:
             return shots_service.get_day_quota_shots(
                 person_id,
@@ -800,6 +740,7 @@ class PersonDayQuotaShotsResource(Resource, ArgsMixin):
                 project_id=project_id,
                 task_type_id=task_type_id,
                 weighted=weighted,
+                feedback=feedback,
             )
         except WrongDateFormatException:
             abort(404)
@@ -813,7 +754,12 @@ class TimeSpentDurationResource(Resource, ArgsMixin):
     def get_person_project_department_arguments(self):
         project_id = self.get_project_id()
         person_id = None
-        department_ids = None
+        department_id = self.get_text_parameter("department_id")
+        if department_id is not None:
+            department_ids = [department_id]
+        else:
+            department_ids = None
+        studio_id = self.get_text_parameter("studio_id")
         if not permissions.has_admin_permissions():
             if (
                 permissions.has_manager_permissions()
@@ -827,9 +773,16 @@ class TimeSpentDurationResource(Resource, ArgsMixin):
                 elif project_id not in project_ids:
                     raise permissions.PermissionDenied
                 if permissions.has_supervisor_permissions():
-                    department_ids = persons_service.get_current_user(
+                    persons_departments = persons_service.get_current_user(
                         relations=True
                     )["departments"]
+                    if department_id is not None:
+                        if department_id not in persons_departments:
+                            raise WrongParameterException(
+                                "Supervisor not allowed to access this department"
+                            )
+                    else:
+                        department_ids = persons_departments
             else:
                 person_id = persons_service.get_current_user()["id"]
 
@@ -837,6 +790,7 @@ class TimeSpentDurationResource(Resource, ArgsMixin):
             "person_id": person_id,
             "project_id": project_id,
             "department_ids": department_ids,
+            "studio_id": studio_id,
         }
 
 
@@ -1300,6 +1254,11 @@ class ChangePasswordForPersonResource(Resource, ArgsMixin):
         permissions.check_admin_permissions()
         try:
             person = persons_service.get_person(person_id)
+            if (
+                person["email"] in config.PROTECTED_ACCOUNTS
+                and person["id"] != persons_service.get_current_user()["id"]
+            ):
+                raise PersonInProtectedAccounts()
             current_user = persons_service.get_current_user()
             auth.validate_password(password, password_2)
             password = auth.encrypt_password(password)
@@ -1343,6 +1302,14 @@ Thank you and see you soon on Kitsu,
             return {"error": True, "message": "Password is too short."}, 400
         except UnactiveUserException:
             return {"error": True, "message": "User is unactive."}, 400
+        except PersonInProtectedAccounts:
+            return (
+                {
+                    "error": True,
+                    "message": "This user is in protected accounts.",
+                },
+                400,
+            )
 
     def get_arguments(self):
         args = self.get_args(
