@@ -3,6 +3,7 @@ from sqlalchemy_utils import UUIDType
 from zou.app import db
 from zou.app.models.serializer import SerializerMixin
 from zou.app.models.base import BaseMixin
+from sqlalchemy.sql import expression
 
 
 class SearchFilter(db.Model, BaseMixin, SerializerMixin):
@@ -15,7 +16,17 @@ class SearchFilter(db.Model, BaseMixin, SerializerMixin):
     entity_type = db.Column(db.String(80))
     name = db.Column(db.String(200), nullable=False, default="")
     search_query = db.Column(db.String(500), nullable=False, default="")
+    is_shared = db.Column(
+        db.Boolean,
+        server_default=expression.false(),
+        default=False,
+        nullable=False,
+    )
 
+    department_id = db.Column(
+        UUIDType(binary=False),
+        db.ForeignKey("department.id"),
+    )
     search_filter_group_id = db.Column(
         UUIDType(binary=False),
         db.ForeignKey("search_filter_group.id"),
