@@ -3,7 +3,7 @@ import uuid
 
 from sqlalchemy.exc import StatementError, IntegrityError, DataError
 from sqlalchemy.sql import func
-from sqlalchemy.sql.expression import case, literal_column
+from sqlalchemy.sql.expression import case
 from sqlalchemy.orm import aliased
 
 from zou.app import app, db
@@ -29,6 +29,7 @@ from zou.app.models.task import Task
 from zou.app.models.task_type import TaskType
 from zou.app.models.task_status import TaskStatus
 from zou.app.models.time_spent import TimeSpent
+from zou.app.models.studio import Studio
 
 from zou.app.utils import (
     cache,
@@ -47,6 +48,7 @@ from zou.app.services.exception import (
     TaskStatusNotFoundException,
     TaskTypeNotFoundException,
     DepartmentNotFoundException,
+    StudioNotFoundException,
     WrongDateFormatException,
     TimeSpentNotFoundException,
 )
@@ -81,16 +83,20 @@ def clear_department_cache(department_id):
     cache.cache.delete_memoized(get_departments)
 
 
+def clear_studio_cache(studio_id):
+    cache.cache.delete_memoized(get_studio, studio_id)
+    cache.cache.delete_memoized(get_studios)
+
+
 def clear_task_cache(task_id):
     cache.cache.delete_memoized(get_task, task_id)
     cache.cache.delete_memoized(get_task, task_id, True)
-    cache.cache.delete_memoized(get_task_with_relations, task_id)
 
 
 @cache.memoize_function(120)
 def clear_comment_cache(comment_id):
     cache.cache.delete_memoized(get_comment, comment_id)
-    cache.cache.delete_memoized(get_comment_with_relations, comment_id)
+    cache.cache.delete_memoized(get_comment, comment_id, True)
 
 
 @cache.memoize_function(120)
@@ -98,6 +104,11 @@ def get_departments():
     return fields.serialize_models(Department.get_all())
 
 
+@cache.memoize_function(120)
+def get_studios():
+    return fields.serialize_models(Studio.get_all())
+
+
 @cache.memoize_function(120)
 def get_task_types():
     return fields.serialize_models(TaskType.get_all())
@@ -160,6 +171,22 @@ def get_department(department_id):
     return department.serialize()
 
 
+@cache.memoize_function(120)
+def get_studio(studio_id):
+    """
+    Get studio matching given id as a dictionary.
+    """
+    try:
+        studio = Studio.get(studio_id)
+    except StatementError:
+        raise StudioNotFoundException
+
+    if studio is None:
+        raise StudioNotFoundException
+
+    return studio.serialize()
+
+
 def get_department_from_task_type(task_type_id):
     """
     Get department of given task type as dictionary
@@ -222,14 +249,6 @@ def get_task(task_id, relations=False):
     return get_task_raw(task_id).serialize(relations=relations)
 
 
-@cache.memoize_function(120)
-def get_task_with_relations(task_id):
-    """
-    Get task matching given id as a dictionary.
-    """
-    return get_task_raw(task_id).serialize(relations=True)
-
-
 def get_task_by_shotgun_id(shotgun_id):
     """
     Get task matching given shotgun id as a dictionary.
@@ -368,7 +387,7 @@ def _convert_rows_to_detailed_tasks(rows, relations=False):
             entity_name,
         ) = entry
 
-        task = get_task_with_relations(str(task_object.id))
+        task = get_task(str(task_object.id), relations=relations)
         task["project_name"] = project_name
         task["task_type_name"] = task_type_name
         task["task_status_name"] = task_status_name
@@ -746,21 +765,11 @@ def get_comment_raw(comment_id):
 
 
 @cache.memoize_function(120)
-def get_comment(comment_id):
+def get_comment(comment_id, relations=False):
     """
     Return comment matching give id as a dict.
     """
-    comment = get_comment_raw(comment_id)
-    return comment.serialize()
-
-
-@cache.memoize_function(120)
-def get_comment_with_relations(comment_id):
-    """
-    Return comment matching give id as a dict with joins information.
-    """
-    comment = get_comment_raw(comment_id)
-    return comment.serialize(relations=True)
+    return get_comment_raw(comment_id).serialize(relations=relations)
 
 
 def get_comment_by_preview_file_id(preview_file_id):
@@ -957,7 +966,7 @@ def get_person_tasks(person_id, projects, is_done=None):
                 except EpisodeNotFoundException:
                     episode_name = "MP"
 
-        task_dict = get_task_with_relations(str(task.id))
+        task_dict = get_task(str(task.id), relations=True)
         if entity_type_name == "Sequence" and entity_parent_id is not None:
             episode_id = entity_parent_id
             episode = shots_service.get_episode(episode_id)
@@ -1054,7 +1063,7 @@ def get_person_tasks_to_check(project_ids=None, department_ids=None):
     else:
         query = query.filter(user_service.build_open_project_filter())
 
-    if department_ids is not None:
+    if department_ids:
         query = query.filter(TaskType.department_id.in_(department_ids))
     tasks = []
     for (
@@ -1089,7 +1098,7 @@ def get_person_tasks_to_check(project_ids=None, department_ids=None):
         if episode_id is None:
             episode_id = entity_source_id
 
-        task_dict = get_task_with_relations(str(task.id))
+        task_dict = get_task(str(task.id), relations=True)
         if entity_type_name == "Sequence" and entity_parent_id is not None:
             episode_id = entity_parent_id
             episode = shots_service.get_episode(episode_id)
@@ -1269,6 +1278,9 @@ def update_task(task_id, data):
     if is_finished(task, data):
         data["end_date"] = date_helpers.get_utc_now_datetime()
 
+    if is_done(task, data):
+        data["done_date"] = date_helpers.get_utc_now_datetime()
+
     task.update(data)
     clear_task_cache(task_id)
     events.emit(
@@ -1462,7 +1474,7 @@ def delete_time_spent(task_id, person_id, date):
 
 def is_finished(task, data):
     """
-    Return True if task status is set to done.
+    Return True if task status is set to feedback request.
     """
     if "task_status_id" in data:
         task_status = get_task_status_raw(task.task_status_id)
@@ -1475,6 +1487,18 @@ def is_finished(task, data):
         return False
 
 
+def is_done(task, data):
+    """
+    Return True if task status is set to done.
+    """
+    if "task_status_id" in data:
+        task_status = get_task_status_raw(task.task_status_id)
+        new_task_status = get_task_status_raw(data["task_status_id"])
+        return new_task_status.id != task_status.id and new_task_status.is_done
+    else:
+        return False
+
+
 def clear_assignation(task_id, person_id=None):
     """
     Clear task assignation and emit a *task:unassign* event.
@@ -1622,20 +1646,14 @@ def update_preview_file_info(preview_file):
         project = projects_service.get_project(task.project_id)
 
         if project["is_set_preview_automated"]:
-            entity_id = str(task.entity_id)
             entity = entities_service.update_entity_preview(
-                entity_id,
+                task.entity_id,
                 preview_file["id"],
             )
-            assets_service.clear_asset_cache(entity_id)
-            edits_service.clear_edit_cache(entity_id)
-            shots_service.clear_shot_cache(entity_id)
-            shots_service.clear_episode_cache(entity_id)
-            shots_service.clear_sequence_cache(entity_id)
     return entity
 
 
-def get_comments_for_project(project_id, page=0):
+def get_comments_for_project(project_id, page=0, limit=None):
     """
     Return all comments for given project.
     """
@@ -1644,7 +1662,9 @@ def get_comments_for_project(project_id, page=0):
         .filter(Task.project_id == project_id)
         .order_by(Comment.updated_at.desc())
     )
-    return query_utils.get_paginated_results(query, page, relations=True)
+    return query_utils.get_paginated_results(
+        query, page, limit, relations=True
+    )
 
 
 def get_time_spents_for_project(project_id, page=0):
@@ -1655,18 +1675,38 @@ def get_time_spents_for_project(project_id, page=0):
     return query_utils.get_paginated_results(query, page)
 
 
-def get_tasks_for_project(project_id, page=0):
+def get_tasks_for_project(
+    project_id, page=0, task_type_id=None, episode_id=None
+):
     """
     Return all tasks for given project.
     """
     query = Task.query.filter(Task.project_id == project_id).order_by(
         Task.updated_at.desc()
     )
+    if task_type_id is not None:
+        query = query.filter(Task.task_type_id == task_type_id)
+    if episode_id is not None:
+        Sequence = aliased(Entity, name="sequence")
+        query = (
+            query.join(Entity, Entity.id == Task.entity_id)
+            .join(Sequence, Sequence.id == Entity.parent_id)
+            .filter(Sequence.parent_id == episode_id)
+        )
+
+        if permissions.has_vendor_permissions():
+            query = query.filter(user_service.build_assignee_filter())
+        elif not permissions.has_admin_permissions():
+            query = query.join(Project).filter(
+                user_service.build_related_projects_filter()
+            )
+        return query
+
     return query_utils.get_paginated_results(query, page, relations=True)
 
 
 def get_full_task(task_id, user_id):
-    task = get_task_with_relations(task_id)
+    task = get_task(task_id, relations=True)
     task_type = get_task_type(task["task_type_id"])
     project = projects_service.get_project(task["project_id"])
     task_status = get_task_status(task["task_status_id"])
@@ -1730,6 +1770,7 @@ def reset_task_data(task_id):
     real_start_date = None
     last_comment_date = None
     end_date = None
+    done_date = None
     entity = entities_service.get_entity(task.entity_id)
     task_status_id = get_default_status(
         for_concept=entity["entity_type_id"]
@@ -1742,6 +1783,7 @@ def reset_task_data(task_id):
         .add_columns(
             TaskStatus.is_retake,
             TaskStatus.is_feedback_request,
+            TaskStatus.is_done,
             TaskStatus.short_name,
         )
         .all()
@@ -1752,6 +1794,7 @@ def reset_task_data(task_id):
         comment,
         task_status_is_retake,
         task_status_is_feedback_request,
+        task_status_is_done,
         task_status_short_name,
     ) in comments:
         if task_status_is_retake and not previous_is_retake:
@@ -1764,6 +1807,11 @@ def reset_task_data(task_id):
         if task_status_is_feedback_request:
             end_date = comment.created_at
 
+        print("ok", task_status_is_done)
+        if task_status_is_done:
+            done_date = comment.created_at
+            print(done_date)
+
         task_status_id = comment.task_status_id
         last_comment_date = comment.created_at
 
@@ -1779,12 +1827,13 @@ def reset_task_data(task_id):
             "real_start_date": real_start_date,
             "last_comment_date": last_comment_date,
             "end_date": end_date,
+            "done_date": done_date,
             "task_status_id": task_status_id,
         }
     )
     project_id = str(task.project_id)
     events.emit("task:update", {"task_id": task.id}, project_id)
-    return task.serialize()
+    return task.serialize(relations=True)
 
 
 def get_persons_tasks_dates():
@@ -1913,14 +1962,20 @@ def get_open_tasks(
         query_stats = query_stats.filter(TaskStatus.id == task_status_id)
 
     if person_id is not None:
-        query = query.filter(Task.assignees.any(id=person_id))
-        query_stats = query_stats.filter(Task.assignees.any(id=person_id))
+        if person_id == "unassigned":
+            query = query.filter(Task.assignees == None)
+            query_stats = query_stats.filter(Task.assignees == None)
+        else:
+            query = query.filter(Task.assignees.any(id=person_id))
+            query_stats = query_stats.filter(Task.assignees.any(id=person_id))
 
     if start_date is not None:
+        start_date = func.cast(start_date, Task.start_date.type)
         query = query.filter(Task.start_date >= start_date)
         query_stats = query_stats.filter(Task.start_date >= start_date)
 
     if due_date is not None:
+        due_date = func.cast(due_date, Task.due_date.type)
         query = query.filter(Task.due_date <= due_date)
         query_stats = query_stats.filter(Task.due_date <= due_date)
 
@@ -1978,7 +2033,7 @@ def get_open_tasks(
                 except EpisodeNotFoundException:
                     episode_name = "MP"
 
-        task_dict = get_task_with_relations(str(task.id))
+        task_dict = get_task(str(task.id), relations=True)
         if entity_type_name == "Sequence" and entity_parent_id is not None:
             episode_id = entity_parent_id
             episode = shots_service.get_episode(episode_id)
@@ -2004,6 +2059,7 @@ def get_open_tasks(
                 "duration": task.duration,
                 "start_date": fields.serialize_value(task.start_date),
                 "due_date": fields.serialize_value(task.due_date),
+                "done_date": fields.serialize_value(task.done_date),
                 "type_name": task_type_name,
                 "task_type_for_entity": task_type_for_entity,
                 "status_name": task_status_name,
@@ -2051,7 +2107,6 @@ def get_open_tasks(
     return result
 
 
-
 def get_open_tasks_stats():
     """
     Return the amount of tasks, done tasks, estimation, and duration for each
@@ -2065,7 +2120,9 @@ def get_open_tasks_stats():
     query_stats = (
         db.session.query(
             func.count().label("amount"),
-            func.count(case({TaskStatus.is_done: Task.id})).label("amount_done"),
+            func.count(case({TaskStatus.is_done: Task.id})).label(
+                "amount_done"
+            ),
             func.sum(Task.duration).label("total_duration"),
             func.sum(Task.estimation).label("total_estimation"),
         )
@@ -2080,7 +2137,7 @@ def get_open_tasks_stats():
         .add_columns(
             Project.id.label("project_id"),
             TaskType.id.label("task_type_id"),
-            TaskStatus.id.label("task_status_id")
+            TaskStatus.id.label("task_status_id"),
         )
     )
 
@@ -2115,7 +2172,7 @@ def get_open_tasks_stats():
                 "amount_done": 0,
                 "total_duration": 0,
                 "total_estimation": 0,
-                "task_types": []
+                "task_types": [],
             }
         project_stats = stats_map[project_id]
         project_stats["amount"] += stat["amount"]
@@ -2124,4 +2181,4 @@ def get_open_tasks_stats():
         project_stats["total_estimation"] += stat["total_estimation"]
         project_stats["task_types"].append(stat)
 
-    return stats_map
\ No newline at end of file
+    return stats_map
